/*
 * help.h
 * Declares functions for displaying program help in multiple languages.
 *
 * Copyright (c) 2004-2006, Dale McCoy.
 * Copyright (c) 2006, Dan Masek.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _RENUM_HELP_H_INCLUDED_
#define _RENUM_HELP_H_INCLUDED_

/*! Display help to standard output. */
void ShowHelp();

#endif // _RENUM_HELP_H_INCLUDED_
