# Current year
YEAR = 2013

ifdef REVISION
VERSIONSTR := r$(REVISION)
else
VERSIONS   := $(shell AWK="$(AWK)" "./findversion.sh")
REV        := $(shell echo "$(VERSIONS)" | cut -f 2 -d'	')
VER        := $(shell echo "$(VERSIONS)" | cut -f 1 -d'	')
VERSIONSTR := $(shell [ "`echo "$(VERSIONS)" | cut -c 1`" = 'r' ] && echo "trunk $(VER)" && exit; echo "$(VER) r$(REV)")
endif
